/****************************************************************************
**
** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
**
** This file is part of the QtGui module of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#ifndef QWIDGET_QWS_P_H
#define QWIDGET_QWS_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists for the convenience
// of other Qt classes.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <qimage.h>
#include <qwsmemid_qws.h>
#include <private/qsharedmemory_p.h>

class QWSLock;

class QWSBackingStore
{
public:
    QWSBackingStore();
    ~QWSBackingStore();

    void create(QSize size, QImage::Format, int windowType);
    bool createIfNecessary(QWidget *tlw);
#ifndef QT_NO_QWS_MULTIPROCESS
    void attach(QWSMemId id, QSize size, QImage::Format, int windowType);
#endif
    void detach();
    void setMemory(QWSMemId id, const QSize &size, QImage::Format, int windowType);

    bool lock(int timeout = -1);
    void unlock();
    bool wait(int timeout = -1);
    void setLock(QWSLock *l);

    const QImage &image() const { return img; }
    QPaintDevice *paintDevice() { return &img; }

    void blit(const QRect &src, const QPoint &dest);

    QWSMemId memoryId() const;
    QSize size() const { return img.size(); }

    bool isNull() const { return img.isNull(); }

    enum WindowType {Transparent=0, Opaque=1, ReservedRegion=2, NonBuffered=3, DebugHighlighter=4, Invalid=5};

    int windowType() const { return _windowType; }

    QPoint tlwOffset() const { return offs; }
private:
    QImage img;
#ifndef QT_NO_QWS_MULTIPROCESS
    QSharedMemory shm;
#endif
    uchar *mem;

    QRegion clipRgn;
    QPoint offs;
    int isSharedMemory : 1;
    int ownsMemory : 1;

    int _windowType;

    QWSLock *memLock;
};

#endif // QWIDGET_QWS_P_H
